"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const chalk = require("chalk");
const replace = require("replace");
const path = require("path");
const fs_1 = require("fs");
const ngast_1 = require("ngast");
const resource_1 = require("./utils/resource");
function translate() {
    const error = message => {
        console.error(chalk.default.bgRed.white(message));
    };
    const info = (message, count1, count2) => {
        console.log(chalk.default.green(message) +
            ` ${count1 ? chalk.default.blue(count1) : ""}` +
            ` ${count2 ? "/ " + chalk.default.yellowBright(count2) : ""}`);
    };
    const args = minimist(process.argv.slice(2));
    validateArgs([args.in, args.out, args.outPath], error);
    let projectPath = args.p;
    const inLocacte = args.in;
    let outLocacte = args.out.split(",");
    const outPath = args.outPath;
    if (!projectPath) {
        projectPath = "./tsconfig.json";
    }
    if (!fs_1.existsSync(projectPath)) {
        error('Cannot find tsconfig at "' + projectPath + '".');
        process.exit(1);
    }
    console.log("Parsing...");
    let parseError = null;
    const projectSymbols = new ngast_1.ProjectSymbols(projectPath, resource_1.resourceResolver, e => (parseError = e));
    let allDirectives = projectSymbols.getDirectives();
    if (!parseError) {
        allDirectives = allDirectives.filter(el => el.symbol.filePath.indexOf("node_modules") === -1);
        const jsonResult = {};
        allDirectives.forEach(el => {
            try {
                if (el.isComponent()) {
                    // Component
                    const moduleName = el.getModule().toSummary().type.reference.name;
                    const componentName = el.symbol.name;
                    const name = moduleName + "." + componentName;
                    if (!jsonResult[moduleName]) {
                        jsonResult[moduleName] = {};
                    }
                    jsonResult[moduleName][componentName] = {};
                    let texts = [];
                    el.getTemplateAst().templateAst.forEach(element => {
                        texts.push(...getTextAst(element));
                    });
                    const url = el.getResolvedMetadata().templateUrl || el.symbol.filePath;
                    texts.forEach((text, i) => {
                        jsonResult[moduleName][componentName][i] = text;
                        replace({
                            regex: text,
                            replacement: `{{ '${name}.${i}' | translate }}`,
                            paths: [url]
                        });
                    });
                }
                else {
                    // Directive
                }
            }
            catch (e) {
                // Component
                error(e);
                // exception only component
            }
        });
        // saving data
        if (!fs_1.existsSync(outPath)) {
            fs_1.mkdirSync(outPath, { recursive: true });
        }
        const locales = [inLocacte, ...outLocacte];
        locales.forEach(locale => {
            const outFile = path.join(outPath, `${locale}.json`);
            fs_1.writeFile(outFile, JSON.stringify(jsonResult), "utf8", () => { });
        });
    }
    else {
        error(parseError);
    }
}
exports.translate = translate;
function getTextAst(element) {
    const texts = [];
    if (element && element.children && element.children.length) {
        element.children.forEach((child) => {
            const value = child
                .value;
            if (value) {
                if (typeof value === "string" && value.trim() !== "") {
                    texts.push(child.value);
                }
                else {
                    const source = value.source;
                    if (typeof value === "object" && source && source.trim() !== "") {
                        texts.push(source);
                    }
                }
            }
            else {
                const childTexts = getTextAst(child);
                childTexts.forEach((el) => {
                    texts.push(el);
                });
            }
        });
    }
    return texts;
}
function validateArgs(args, error) {
    args.forEach(arg => {
        if (!arg || arg.trim().length === 0) {
            error(`Connot find --${arg} argument`);
            process.exit(1);
        }
    });
}
//# sourceMappingURL=translate.js.map